package com.redhat.thermostat.tools.eclipse.plugin.editor;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.redhat.thermostat.tools.eclipse.plugin.BundleFinder;
import com.redhat.thermostat.tools.eclipse.plugin.BundleInformation;
import com.redhat.thermostat.tools.eclipse.plugin.BundleInformationExtractor;
import com.redhat.thermostat.tools.eclipse.plugin.Messages;

/**
 * Prompt the user to enter or select a bundle name/version.
 *
 * Used by the command edit page.
 */
public class NewBundleDialog extends TitleAreaDialog {

    private BundleInformation bundleInfo = new BundleInformation();

    private ThermostatBundleInformation thermostatInfo = new ThermostatBundleInformation();
    private BundleFinder projectBundleFinder;

    private Button projectBundlesSelector;
    private Combo projectBundles;
    private ComboViewer projectBundlesViewer;

    private Button thermostatBundlesSelector;
    private Combo thermostatBundleVersion;
    private Combo thermostatBundleName;

    private Button manualBundleSelector;
    private Text manualBundleName;
    private Text manualBundleVersion;

    private Button manualJarSelector;
    private Text manualJarLocation;
    private Button manualJarFindFile;

    public NewBundleDialog(Shell parentShell) {
        super(parentShell);

        projectBundleFinder = new BundleFinder(getOpenProjects(), new NullProgressMonitor());
    }

    private IProject[] getOpenProjects() {
        IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();

        IProject[] allProjects = root.getProjects();
        List<IProject> openProjects = new ArrayList<>();
        for (IProject project: allProjects) {
            if (project.isOpen()) {
                openProjects.add(project);
            }
        }

        return openProjects.toArray(new IProject[openProjects.size()]);
    }

    @Override
    public void create() {
        super.create();

        setTitle(Messages.NewBundleDialog_title);
        setMessage(Messages.NewBundleDialog_message);
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite area = (Composite) super.createDialogArea(parent);

        Composite container = new Composite(area, SWT.NONE);
        container.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,true));

        GridLayout containerLayout = new GridLayout(3, false);
        containerLayout.marginTop = 5;
        containerLayout.marginRight = 5;
        containerLayout.marginLeft = 5;
        container.setLayout(containerLayout);

        projectBundlesSelector = new Button(container, SWT.RADIO);
        projectBundlesSelector.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                disableInputWidgets();

                projectBundles.setEnabled(true);
            }
        });
        projectBundlesSelector.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 3, 1));
        projectBundlesSelector.setText("Current workspace projects");

        Label projectBundlesLabel = new Label(container, SWT.None);
        projectBundlesLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
        projectBundlesLabel.setText("Bundle");

        projectBundles = new Combo(container, SWT.READ_ONLY | SWT.DROP_DOWN);
        GridData projectBundlesLayoutData = new GridData(SWT.FILL, SWT.BEGINNING, true, false, 2, 1);
        projectBundles.setLayoutData(projectBundlesLayoutData);

        projectBundlesViewer = new ComboViewer(projectBundles);
        projectBundlesViewer.setContentProvider(new ArrayContentProvider());
        projectBundlesViewer.setInput(projectBundleFinder.find());
        projectBundlesViewer.setLabelProvider(new ColumnLabelProvider() {
            @Override
            public String getText(Object element) {
                BundleInformation bundle = (BundleInformation) element;
                return bundle.getName() + " = " + bundle.getVersion();
            }
        });

        thermostatBundlesSelector = new Button(container, SWT.RADIO);
        thermostatBundlesSelector.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                disableInputWidgets();

                thermostatBundleVersion.setEnabled(true);
                thermostatBundleName.setEnabled(true);
            }
        });
        thermostatBundlesSelector.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 3, 1));
        thermostatBundlesSelector.setText("Thermostat Bundle");

        Label thermostatVersionLabel = new Label(container, SWT.None);
        thermostatVersionLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
        thermostatVersionLabel.setText("Version");

        thermostatBundleVersion = new Combo(container, SWT.None);
        GridData thermostatVersionLayoutData = new GridData(SWT.FILL, SWT.BEGINNING, true, false, 2, 1);
        thermostatBundleVersion.setLayoutData(thermostatVersionLayoutData);
        thermostatBundleVersion.setItems(toArray(thermostatInfo.getVersions()));
        thermostatBundleVersion.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                String selectedVersion = thermostatBundleVersion.getItem(thermostatBundleVersion.getSelectionIndex());
                thermostatBundleName.setItems(toArray(thermostatInfo.getBundleNames(selectedVersion)));
            }
        });

        Label thermostatBundleLabel = new Label(container, SWT.None);
        thermostatBundleLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
        thermostatBundleLabel.setText("Bundle");

        thermostatBundleName = new Combo(container, SWT.NONE);
        GridData thermostatBundleLayoutData = new GridData(SWT.FILL, SWT.BEGINNING, true, false, 2, 1);
        thermostatBundleName.setLayoutData(thermostatBundleLayoutData);

        manualBundleSelector = new Button(container, SWT.RADIO);
        manualBundleSelector.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                disableInputWidgets();

                manualBundleName.setEnabled(true);
                manualBundleVersion.setEnabled(true);
            }
        });
        manualBundleSelector.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 3, 1));
        manualBundleSelector.setText(Messages.NewBundleDialog_manualSectionTitle);

        Label nameLabel = new Label(container, SWT.None);
        nameLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
        nameLabel.setText(Messages.NewBundleDialog_symbolicNameLabel);
        manualBundleName = new Text(container, SWT.None);
        GridData nameTextLayoutData = new GridData(SWT.FILL, SWT.BEGINNING, true, false);
        nameTextLayoutData.horizontalSpan = 2;
        manualBundleName.setLayoutData(nameTextLayoutData);

        Label versionLabel = new Label(container, SWT.None);
        versionLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
        versionLabel.setText(Messages.NewBundleDialog_versionLabel);
        manualBundleVersion = new Text(container, SWT.None);
        GridData versionTextLayoutData = new GridData(SWT.FILL, SWT.BEGINNING, true, false);
        versionTextLayoutData.horizontalSpan = 2;
        manualBundleVersion.setLayoutData(versionTextLayoutData);

        manualJarSelector = new Button(container, SWT.RADIO);
        manualJarSelector.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                disableInputWidgets();

                manualJarLocation.setEnabled(true);
                manualJarFindFile.setEnabled(true);
            }
        });
        manualJarSelector.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 3, 1));
        manualJarSelector.setText(Messages.NewBundleDialog_fromJarSectionTitle);

        Label locationLabel = new Label(container, SWT.NONE);
        locationLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
        locationLabel.setText(Messages.NewBundleDialog_pathLabel);

        manualJarLocation = new Text(container, SWT.BORDER);
        manualJarLocation.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        manualJarFindFile = new Button(container, SWT.NONE);
        manualJarFindFile.setText(Messages.NewBundleDialog_findOnDisk);
        manualJarFindFile.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                FileDialog dialog = new FileDialog(getShell(), SWT.OPEN);
                dialog.setFilterExtensions(new String[] { "*.jar" }); //$NON-NLS-1$
                String result = dialog.open();
                if (result != null) {
                    manualJarLocation.setText(result);
                }
            }
        });

        // invoke manually on startup to enable/disable other widgets
        projectBundlesSelector.setSelection(true);

        return area;
    }

    private void disableInputWidgets() {
        projectBundles.setEnabled(false);

        thermostatBundleVersion.setEnabled(false);
        thermostatBundleName.setEnabled(false);

        manualBundleName.setEnabled(false);
        manualBundleVersion.setEnabled(false);

        manualJarLocation.setEnabled(false);
        manualJarFindFile.setEnabled(false);
    }

    private String[] toArray(List<String> inputList) {
        return inputList.toArray(new String[inputList.size()]);
    }


    @Override
    protected void okPressed() {
        saveInput();
        super.okPressed();
    }

    private void saveInput() {
        if (projectBundlesSelector.getSelection()) {
            IStructuredSelection selection = (IStructuredSelection) projectBundlesViewer.getSelection();
            bundleInfo = (BundleInformation) selection.getFirstElement();
        } else if (thermostatBundlesSelector.getSelection()) {
            bundleInfo = new BundleInformation(thermostatBundleName.getText(), thermostatBundleVersion.getText());
        } else if (manualBundleSelector.getSelection()) {
            bundleInfo = new BundleInformation(manualBundleName.getText(), manualBundleVersion.getText());
        } else if (manualJarSelector.getSelection()) {
            try {
                BundleInformationExtractor extractor = new BundleInformationExtractor();
                BundleInformation extractedInfo = extractor.extract(manualJarLocation.getText());
                bundleInfo.setName(extractedInfo.getName());
                bundleInfo.setVersion(extractedInfo.getVersion());
            } catch (final IOException e) {
                getShell().getDisplay().asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        MessageDialog.openError(getShell(), Messages.NewBundleDialog_errorExtractingInformationFromJarFile, e.getMessage());
                    }
                });
            }
        } else {
            throw new AssertionError("Unexpected case");
        }
    }

    public BundleInformation getBundleInformation() {
        return bundleInfo;
    }
}
